local WMS = WMS_016

local UpgradeProto = {
    name = "Error: Base has no name.",
    conditions = { invalidCounter = -1 },
    childWidgetToEnable = "Error: no widget specified",
    
--~     changedCounter (optional): Pass a recently changed counter to determine wether 
--~     that change caused an achievement.
--~     Returns 1: Achievement has been achieved in the past
--~     Returns 0: Achievement happend just right now, because changedCounter reached the condition
--~     Returns -1: Achievement not yet achieved
    achievedCheck = function(self, changedCounter)
        local retVal = 1 -- assume achievement achieved until a condition is found that contradicts
        for counter, threshold in pairs(self.conditions) do
            local conCount = WMS:GetConCount(counter)
            if (conCount < threshold) then
--~                 print(self.name, " ", counter , ": ", conCount, " < threshold ", threshold)
                retVal = -1 -- achievement not achieved yet
                break
            elseif (conCount == threshold) and (counter == changedCounter) then
                retVal = 0  -- achievement achieved just right now
            end
        end
        return retVal
    end,
    
--~     Turns on/off the widget that is enabled by this achievement
    applyUpgrade = function(self)
        local child = WMS:GetChildWidget(self.childWidgetToEnable)
        if child then
            local achieved = self:achievedCheck() 
--~             print("Achievement ", self.name,": ", achieved)
            if achieved >= 0 then
                child:Show()
            else
                child:Hide()
            end
        end
    end
}

local function CreateUpgrade(name, conditions, childWidgetToEnable)
    local upgrade = {}
    setmetatable(upgrade, {__index = UpgradeProto})
    upgrade.name = name
    upgrade.conditions = conditions
    upgrade.childWidgetToEnable = childWidgetToEnable
    return upgrade
end

WMS.Upgrades = {
    CreateUpgrade(
        "WMS-016 Send Button",
        {[WMS.MSG_CODES.CLEAR_TO_SEND] = 1},
        "SendMsgButton"
    ),
    CreateUpgrade(
        "WMS-016 1st Receipt",
        {[WMS.MSG_CODES.RECEIVE] = 1},
        nil
    ),
    CreateUpgrade(
        "WMS-016 1st Transmission",
        {[WMS.MSG_CODES.TRANSMIT] = 1},
        nil
    ),
    CreateUpgrade(
        "WMS-016 Transfer Engine",
        {
            [WMS.MSG_CODES.TRANSMIT] = 5,
            [WMS.MSG_CODES.RECEIVE] = 5
        },
        "BufferTransferButton"
    ),
    CreateUpgrade(
        "WMS-016 Redirection Stats",
        {
            [WMS.MSG_CODES.BUFFER_TRANSFER] = 5,
        },
        "Redirector"
    ),
    CreateUpgrade(
        "WMS-016 Crypter",
        {
            [WMS.MSG_CODES.NOTIFY_REDIRECTED] = 5,
        },
        "Crypter"
    )
}

function WMS:CheckForUpgrade(counterType)
    for _, upgrade in pairs(self.Upgrades) do
        upgrade:applyUpgrade()
        if upgrade:achievedCheck(counterType) == 0 then
            self:UpgradeAlert(upgrade.name)
        end
    end
end

function WMS:UpgradeAlert(name)   
    local achievementFrm = WMS.achivementFrame
    local title = _G[achievementFrm:GetName() .. "Title"]
    title:SetText(TL("Achievement Earned"))
    
    local text = _G[achievementFrm:GetName() .. "Text"]
    text:SetText(TL(name))
    achievementFrm.fade:Stop();
    achievementFrm:Hide();
    achievementFrm:Show();
    PlaySoundFile("Sound\\INTERFACE\\LevelUp.ogg", "sfx")
    
    local chatMsg = TL("has earned the achievement:").. " "..TL(name)
    SendChatMessage(chatMsg, "EMOTE");
end

function WMS.AchivementFrame_OnLoad(frame)
    WMS.achivementFrame = frame
end
